
# q-expansion of newform 729.2.e.c, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 729
weight = 2
poly_data = [1, 0, 0, -1, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2, [0, 0, 0, 0, 0, -1]]]
aps_data = [[0, 1, 1, -1, -1, 0], [0, 0, 0, 0, 0, 0], [0, 0, -2, 1, -1, 2], [-1, 0, 1, 2, -1, 1], [-1, -1, -3, -2, 0, 2], [1, -2, -1, -2, 1, 0], [-3, 0, 0, 3, 0, 0], [0, 0, 3, 1, 3, 0], [-1, -2, 0, 3, 3, -2], [-3, 4, -2, 2, -4, 3], [1, 1, 0, -2, -2, 4], [1, -3, 3, -1, 0, 0], [-5, 2, -1, 2, -5, 0], [-1, 4, 1, 2, 0, -2], [-5, 0, 5, 0, -2, -5], [6, 3, 3, 0, 0, -3], [0, 0, 7, 1, -1, -7], [-1, 0, 1, 5, 2, 4], [-5, 4, 2, 4, -5, 0], [3, -6, 9, -3, -3, -3], [0, -6, 3, -2, 3, -6], [-4, 2, 1, -1, -2, 4], [3, -2, 4, -4, 2, -3], [0, -9, 3, 0, 3, -9], [-4, -5, 4, 8, 0, -8], [4, 0, -4, 3, 4, 7], [4, 4, -5, 4, -8, 5], [3, -3, -3, 0, 0, 3], [-10, 0, 0, 0, 3, -3], [-6, -6, -5, 1, 5, 5], [0, -3, 0, 1, 0, -3], [-13, 7, 0, 6, 6, 4], [-6, 7, -5, 5, -7, 6], [-2, 7, 0, -5, -5, -5], [4, 5, 5, 5, 4, 0], [-1, -8, -5, -4, 0, 4], [10, 10, 7, -11, 1, -7], [-1, 12, 12, 0, -3, -9], [-3, -3, 1, 1, 2, -1], [-4, -7, 3, 7, 0, -7], [3, -6, 0, -3, 6, 6], [0, 6, -9, 1, -9, 6], [3, -11, 4, -4, 11, -3], [-5, 1, 0, 4, 4, 10], [0, 0, 6, -12, 6, 0], [1, 0, 6, -1, -6, -6], [4, 4, 7, 1, -5, -7], [-7, 0, 7, -4, -1, -11], [5, 11, 3, -2, 0, 2], [10, -2, -10, -2, 10, 0], [-3, 0, -9, 3, 9, 9], [8, -2, 0, -6, -6, 16], [-1, 2, -11, 11, -2, 1], [0, 9, -3, 15, -3, 9], [-5, 11, -1, 11, -5, 0], [-5, 0, 5, 3, 13, -2], [9, -9, -9, 0, 0, 9], [-7, -6, -6, 0, 3, 3], [-13, 0, 13, 17, -1, 4], [-4, 2, 12, 16, 0, -16], [7, -14, -1, -14, 7, 0], [-7, 16, 0, -9, -9, -8], [-11, -15, 12, 11, 3, 3], [4, -10, 2, -10, 4, 0], [-1, -5, 1, 2, 0, -2], [4, 0, -4, 0, 25, 4], [20, 0, -20, -4, -1, 16], [-8, -2, 8, -2, -8, 0], [-1, 7, 0, -6, -6, -11], [-4, 20, -8, 8, -20, 4], [9, -17, 1, -1, 17, -9], [0, -9, 0, -15, 0, -9], [-1, 19, -5, -4, 0, 4], [-11, -11, -11, -5, 16, 11], [-1, 15, 15, 0, -18, 3], [3, 3, -5, 10, -13, 5], [2, -4, -9, -11, 0, 11], [0, 9, -15, -8, -15, 9], [-4, -2, 0, 6, 6, 4], [-8, 1, 0, 7, 7, 4], [-14, 14, 14, 14, -14, 0], [11, 4, 16, 5, 0, -5], [3, 15, 15, 0, 9, -24], [-19, -6, -6, 0, 15, -9], [14, 0, -14, -16, 2, -2], [5, -16, 12, 7, 0, -7], [18, -12, -9, -18, 21, 21], [-7, -7, -5, 5, 7, 7], [6, 13, 16, -16, -13, -6], [22, -5, 0, -17, -17, -5], [0, 9, -12, -21, -12, 9], [13, 0, -13, -15, -14, -2], [8, -3, -3, 0, -9, 12], [6, 6, -8, -8, 2, 8], [7, -2, 2, -2, 7, 0], [-24, 3, 0, 24, -3, -3], [-4, -11, 0, 15, 15, 4], [0, 0, 3, 27, 3, 0], [-2, 3, 12, 2, -15, -15], [17, -18, -18, 0, 3, 15], [5, 0, -5, -7, -16, -2], [-15, -15, 9, 15, 6, 6], [-4, -11, 0, 15, 15, -14], [-15, -2, -23, 23, 2, 15], [-11, -2, 0, 13, 13, -23], [-2, 3, -15, 2, 12, 12], [22, 0, -22, -12, -17, 10], [6, 12, 12, 0, 9, -21], [6, 6, 1, 1, -7, -1], [2, 0, -2, 5, -19, 7], [-20, 4, -10, 4, -20, 0], [0, 3, -6, -11, -6, 3], [-1, -11, 0, 12, 12, 7], [23, -13, 16, -16, 13, -23], [-8, 12, -27, 8, 15, 15], [-14, 0, 14, 12, -5, -2], [-17, -17, 13, 10, 7, -13], [3, 6, 6, 0, -9, 3], [-15, -15, -23, 19, -4, 23], [17, -1, 6, -11, 0, 11], [10, 10, 2, 10, 10, 0], [11, -16, 4, -4, 16, -11], [18, 1, 19, -19, -1, -18], [0, -9, 12, 9, 12, -9], [32, 13, 13, -19, 0, 19], [15, -24, -24, 0, 0, 24], [2, 0, -2, -13, -1, -11], [21, 12, 0, -21, -12, -12], [2, 14, 10, -10, -14, -2], [16, -2, 0, -14, -14, 13], [19, 0, 15, -19, -15, -15], [-14, 14, 23, 14, -14, 0], [13, 13, 25, 10, -23, -25], [-25, -6, -6, 0, -15, 21], [9, 9, 25, 10, -19, -25], [16, -5, -10, -5, 16, 0], [0, 6, 27, 0, -33, -33], [1, -20, 0, 19, 19, -8], [4, -19, -16, -19, 4, 0], [-18, 9, 9, 0, -9, 0], [17, -3, -3, 0, 0, 3], [-7, -13, 9, 16, 0, -16], [-5, 1, 17, 1, -5, 0], [9, 15, -18, -9, 3, 3], [0, -24, 6, -8, 6, -24], [-21, 4, -11, 11, -4, 21], [-4, -17, -11, -7, 0, 7], [4, 0, -4, -3, -17, 1], [-14, -14, -29, 10, 4, 29], [3, -12, -12, 0, -9, 21], [-5, 10, -10, 10, -5, 0], [-3, 18, 0, 3, -18, -18], [0, -24, 24, 1, 24, -24], [2, -11, 0, 9, 9, 1], [-4, 4, 4, 8, 0, -8], [-5, 0, 5, -9, -2, -14], [11, -6, -6, 0, -6, 12], [11, 23, 0, -11, 0, 11], [0, 15, -9, -8, -9, 15], [5, 16, 0, -21, -21, 22], [-9, 10, 10, -10, -10, 9], [0, 18, -18, 3, -18, 18], [4, 4, -23, 13, -17, 23], [24, 3, 3, 0, -18, 15], [18, 18, 16, 1, -19, -16], [2, 14, 0, -2, 0, 2], [0, 0, 0, -2, 0, 0], [-28, 5, -23, 23, -5, 28]]
